/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.workflows

import groovy.util.logging.Slf4j
import javax.sql.DataSource
import metridoc.plugins.datasource.DataSourcePlugin
import metridoc.utils.DataSourceUtils

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 2/22/12
 * Time: 12:22 PM
 */
@Slf4j
class DataSourceModule extends Script {

    @Override
    Object run() {
        if (binding.variables.containsKey("projectName") && projectName != null) {
            dataSource = doCreateDataSource(projectName)
        }

        createDataSource = {String name ->
            doCreateDataSource(name)
        }

        createEmbeddedDataSource = {
            DataSourceUtils.embeddedDataSource()
        }
    }

    DataSource doCreateDataSource(String dataSourceName) {
        DataSource result = null
        log.info "loading dataSource for ${dataSourceName}"
        if (binding.variables.containsKey(dataSourceName)) {
            def variable = binding.getVariable(dataSourceName)
            if (variable instanceof ConfigObject && variable.containsKey("dataSource")) {
                def dataSourceConfig = variable.dataSource
                if (dataSourceConfig instanceof DataSource) {
                    result = dataSourceConfig
                    log.debug("found dataSource based on direct assignment")
                }

                if (dataSourceConfig instanceof ConfigObject) {
                    use(DataSourcePlugin) {
                        result = dataSource(user: dataSourceConfig.user, password: dataSourceConfig.password, url: dataSourceConfig.url)
                    }
                    log.debug("found dataSource based on properties")
                }
            }
        }

        return result
    }
}

