/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.workflows

import groovy.util.logging.Slf4j
import metridoc.workflows.validation.impl.FileIngestorValidationErrorHandler
import metridoc.workflows.validation.impl.HashSqlValidator
import org.apache.camel.Exchange

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 2/28/12
 * Time: 2:04 PM
 */
@Slf4j
class ValidatorModule extends Script {

    @Override
    def run() {

        log.info("creating the bean 'mapValidator' which is an instance of {}", HashSqlValidator)
        mapValidator(HashSqlValidator)



        log.info("creating the function 'mapSqlValidator' which validates records going to an sql database")
        validateMap = {Exchange record ->
            try {
                mapValidator.validate(record.in.getBody(Map.class))
                return true
            } catch (AssertionError error) {
                log.warn("VALIDATION ERROR: {}", error.message)
                validationErrorHandler.handle(error, record)
                return false
            }
        }

        validationErrorHandler(FileIngestorValidationErrorHandler) {
            validationErrorScope = projectName
            beanConfig.initMethod = "updateSchema"
        }
    }

    boolean bindingContains(String name) {
        binding.variables.containsKey(name)
    }
}

