/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.workflows

import metridoc.workflows.builder.ConfigBuilder
import metridoc.workflows.builder.MetridocBeanBuilder

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 2/15/12
 * Time: 10:40 AM
 */
class WorkflowRunner extends Script {
    String pipelineName = "pipeline"
    List modules
    ConfigBuilder configBuilder = new MetridocBeanBuilder()
    private boolean hasConfigured = false
    List pipeline

    @Override
    Object run() {
        def pipelineToRun = getPipeline()
        pipelineToRun.each {
            it.run()
        }
    }

    synchronized void configure() {
        if (!hasConfigured) {
            configBuilder.load(modules)
            hasConfigured = true
        }
    }

    List getPipeline() {
        if (pipeline) {
            return pipeline
        }
        configure()
        pipeline = configBuilder.getPipeline(pipelineName)
    }

    /**
     *
     * will use {@link WorkflowRunner#createWorkflow} instead
     *
     * @deprecated
     *
     * @param closure
     * @return
     */
    public static WorkflowRunner configure(Closure closure) {
        def result = new WorkflowRunner()
        result.configBuilder.configure(closure)

        return result
    }

    public static WorkflowRunner createWorkflow(Closure closure) {
        createWorkflow(null, closure)
    }

    public static WorkflowRunner createWorkflow(Class<Script> module, Closure closure) {
        def result = new WorkflowRunner()
        result.configBuilder.configure {
            if (module != null) {
                importModule module
            }
            closure.call()
        }

        return result
    }
}
