/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.workflows.operations.impl

import com.jcraft.jsch.ChannelSftp
import com.jcraft.jsch.JSch
import com.jcraft.jsch.Session
import groovy.util.logging.Slf4j
import metridoc.utils.SystemUtils
import metridoc.workflows.operations.FileHandler
import metridoc.workflows.operations.RemoteFiles

/**
 * Created by IntelliJ IDEA.
 * User: tbarker
 * Date: 2/12/12
 * Time: 1:19 PM
 *
 * @deprecated
 */
@Slf4j
class SftpFiles extends RemoteFiles {
    String ssh = "${SystemUtils.USER_HOME}/.ssh"
    String keyFile = "${ssh}/id_rsa"
    String remoteDirectory
    String knownHost = "${ssh}/known_hosts"
    String username
    String tempDirectory = "${SystemUtils.USER_HOME}/.metridoc/tmp"
    String password
    String host
    boolean strictHostKeyChecking = true
    int port = 22
    JSch jsch = new JSch();
    private ChannelSftp channel
    private Session session
    private List<String> _fileNames = []

    @Override
    FileHandler get(int index) {
        def fileNames = getFileNames()
        validateIndex(index, fileNames.size())
        def fileName = getFileNames()[index]
        transfer(fileName)
        def fullLocalFileName = getFullPathToLocalFile(fileName)

        return new SystemFileHandler(file: new File(fullLocalFileName))
    }

    protected void transfer(String fileName) {
        new File(getFilePath(tempDirectory, remoteDirectory)).mkdirs()
        def localDestination = getFullPathToLocalFile(fileName)

        if (shouldTransfer(localDestination)) {
            log.info("copying remote file {} to {}", fileName, localDestination)
            getChannel().get(fileName, localDestination)
            def destinationFile = new File(localDestination)
            destinationFile.deleteOnExit()
        }
    }

    protected boolean shouldTransfer(String fileName) {
        return shouldTransferBasedOnFullPath(
            getFullPathToLocalFile(fileName)
        )
    }

    protected String getFullPathToLocalFile(String fileName) {
        return getFilePath(tempDirectory, getFilePath(remoteDirectory, fileName))
    }

    /**
     * checks if the file has already been transferred
     * @param fullPath
     * @return
     */
    protected static boolean shouldTransferBasedOnFullPath(String fullPath) {
        return !new File(fullPath).exists()
    }

    protected static String getFilePath(String base, String fileName) {
        def slash = SystemUtils.FILE_SEPARATOR
        if (fileName.startsWith(slash)) {
            fileName = fileName.substring(1)
        }
        return "${base}${slash}${fileName}"
    }

    protected static void validateIndex(int index, int maxSize) {
        if (index >= maxSize) {
            throw new ArrayIndexOutOfBoundsException(index)
        }
    }

    ChannelSftp getChannel() {
        if (channel) {
            return channel
        }

        channel = getSession().openChannel("sftp") as ChannelSftp
        channel.connect()
        channel.cd(remoteDirectory)
        return channel
    }

    Session getSession() {
        if (session) {
            return session
        }

        if (knownHost && new File(knownHost).exists()) {
            jsch.setKnownHosts(knownHost)

        }

        if (keyFile && new File(keyFile).exists()) {
            jsch.addIdentity(keyFile)
        }

        session = jsch.getSession(username, host, port)

        if (!strictHostKeyChecking) {
            session.setConfig("StrictHostKeyChecking", "no")
        }


        if (password) {
            session.setPassword(password)
        }

        session.connect()

        return session
    }

    @Override
    List<String> getFileNames() {

        if (_fileNames) {
            return _fileNames
        }

        def channel = getChannel()
        def files = channel.ls(".")
        def result = []

        files.each {ChannelSftp.LsEntry entry ->
            if (!entry.attrs.isDir()) {
                result.add entry.filename
            }
        }

        _fileNames = result.sort()
    }

    @Override
    public String toString() {

        String result
        try {
            result = "SftpFiles{" +
                "fileNames=" + getFileNames() +
                '}';
        } catch (Exception e) {
            log.warn "exception occurred trying to call ${SftpFiles}.toString", e
            result = "SftpFiles{" +
                "fileNames = [unknown]"
            '}';
        }

        return result
    }


}
