/*
 * Copyright 2010 Trustees of the University of Pennsylvania Licensed under the
 * Educational Community License, Version 2.0 (the "License"); you may
 * not use this file except in compliance with the License. You may
 * obtain a copy of the License at
 *
 * http://www.osedu.org/licenses/ECL-2.0
 *
 * Unless required by applicable law or agreed to in writing,
 * software distributed under the License is distributed on an "AS IS"
 * BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express
 * or implied. See the License for the specific language governing
 * permissions and limitations under the License.
 */
package metridoc.workflows.operations.impl

import groovy.util.logging.Slf4j
import metridoc.workflows.operations.DirectoryOperations
import metridoc.workflows.operations.FileHandler

/**
 * @deprecated
 */
@Slf4j
class SystemDirectoryOperations implements DirectoryOperations {

    public static final String DIRECTORY_IS_NULL = "the workingDirectory cannot be null"
    String directory
    String fileNameFilter
    Closure fileHandlerFilter

    List<String> listFilteredFileNames() {
        assert directory: DIRECTORY_IS_NULL
        return getFilteredFileNames()
    }

    protected List<String> getFilteredFileNames() {
        log.debug("retrieving file names from workingDirectory {}", directory)
        def unfiltered = []
        new File(directory).listFiles().each {
            unfiltered.add it.name
        }
        getFilteredFileNames(unfiltered)
    }

    protected List<String> getFilteredFileNames(List<String> unfilteredFileNames) {
        def result = []
        unfilteredFileNames.each {
            if (acceptFile(it)) {
                result.add(it)
            }
        }

        return result
    }

    protected boolean acceptFile(String name) {
        return !fileNameFilter || name =~ fileNameFilter
    }

    List<FileHandler> getFiles() {
        assert directory: DIRECTORY_IS_NULL
        def result = []
        log.debug("retrieving files from workingDirectory {}", directory)
        Set<String> legalFileNames = listFilteredFileNames() as Set
        def unfilteredFiles = getUnFilteredFiles()
        log.debug("testing if files {} should be processed", unfilteredFiles)
        unfilteredFiles.each {
            log.debug("testing if file {} should be processed", it)
            def legalFileName = legalFileNames.contains(it.name)

            if (legalFileName) {
                if (fileHandlerFilter) {
                    if (fileHandlerFilter(it)) {
                        result.add(it)
                        log.debug("file {} will be processed", it)
                    }
                } else {
                    result.add(it)
                    log.debug("file {} will be processed", it)
                }
            } else {
                log.debug("file {} will not be processed since its name is not legal", it)
            }
        }
        return result
    }

    protected List<FileHandler> getUnFilteredFiles() {
        assert directory: DIRECTORY_IS_NULL
        def result = []
        new File(directory).listFiles().each {
            result.add new SystemFileHandler(file: it)
        }

        return result
    }
}